/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/**
 * Interface for a circular sequence, a sequence without head or tail
 * and without the possibility of boundary violations in <code>prev(.)</code>
 * and <code>next(.)</code>.  <b>Note:</b> After splicing a
 * <code>CircularSequence</code> into another sequence, the container that was
 * spliced becomes invalid, and will throw an
 * <code>InvalidContainerException</code> if any method is called on it.
 *   
 * @author Mark Handy
 * @author Mike Boilen (mgb)
 * @version JSDL-Teach release 1.0
 * @see splice
 */

public interface CircularSequence extends PositionalContainer {

	/** 
	 * Gets the <code>Position</code> before a given <code>Position</code> in
	 * this <code>CircularSequence</code>
	 * 
	 * @param p <code>Position</code> of element in this sequence
	 * @return <code>Position</code> of previous element
	 * @exception InvalidPositionException if <code>Position</code> is
	 * invalid, from a different container, or <code>null</code>.
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Position before (Position p) throws
		InvalidPositionException;

	/** 
	 * Gets the <code>Position</code> after a given <code>Position</code> in
	 * this <code>CircularSequence</code>
	 * 
	 * @param p <code>Position</code> of element in this sequence
	 * @return <code>Position</code> of next element
	 * @exception InvalidPositionException if <code>Position</code> is
	 * invalid, from a different container, or <code>null</code>.
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Position after (Position p) throws
		InvalidPositionException;

	/** 
	 * Since there's no <code>first()</code> or <code>last()</code> in a circular
	 * sequence, a method is needed that will give a position -- any position --
	 * in the circular sequence, to get things started.  No guarantees about what
	 * position is returned.
	 *
	 * @return An arbitrary position in the circular sequence
	 * @exception EmptyContainerException if the container is empty
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Position startingPosition() throws EmptyContainerException;


	// Three insertion methods that take an Object element, make a new
	// position, and return that position

	/** 
		* Inserts an element before a given position.
	 * 
	 * @param p <code>Position</code> of element in this sequence
	 * @param element Any <code>java.lang.Object</code>
	 * @return <code>Position</code> of <code>Object</code> element, now stored
	 * ahead of <code>Position p</code> (the parameter)
	 * @exception InvalidPositionException if <code>p</code> is <code>null</code>
	 * or not in this <code>CircularSequence</code>
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Position insertBefore (Position p, Object element)
		throws InvalidPositionException;

	
	/** 
		* Inserts an element after a given position.
	 * 
	 * @param p <code>Position</code> of element in this sequence
	 * @param element Any <code>java.lang.Object</code>
	 * @return <code>Position</code> of <code>Object</code> element, now stored
	 * ahead of <code>Position p</code> (the parameter)
	 * @exception InvalidPositionException if <code>p</code> is <code>null</code>
	 * or not in this <code>CircularSequence</code>
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */

	public Position insertAfter (Position p, Object element)
		throws InvalidPositionException;

	/** 
	 * Inserts an element into an empty <code>CircularSequence</code>. The
	 * counterpart to startingPosition:  intended for use only in an empty
	 * circular sequence, where there is no position before or after which you
	 * can insert.  
	 *
	 * @param element Any <code>Object</code>
	 * @return <code>Position</code> of the object inserted
	 * @exception NotEmptyContainerException if this sequence is not empty.
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Position insertInitial (Object element) throws NotEmptyContainerException;


	/**
	 * Removes the specified position.
	 *
	 * @param p The <code>Position</code> of the element after the element to
	 * be removed.  
	 * @return The removed element.
	 * @exception EmptyContainerException If there are no elements in this
	 * container.
	 * @exception InvalidPositionException if <code>p</code> is
	 * <code>null</code> or not in this <code>CircularSequence</code>
	 * @exception InvalidContainerException if this container has been
	 * invalidated.
	 * @see splice
	 */
	public Object remove (Position p)
		throws EmptyContainerException,
	InvalidPositionException;




	/** 
	 * Snips out a chain of nodes from this <code>CircularSequence,</code>
	 * repairs this <code>CircularSequence,</code> and makes a new
	 * <code>CircularSequence</code> with the snipped-out nodes.  If
	 * <code>inNewA==inNewB,</code> only that one Position is removed from this
	 * <code>CircularSequence.</code> @param inNewA Start of sequence of
	 * <code>Positions</code> to be removed from this
	 * <code>CircularSequence</code>.  In structures that can implement this
	 * method in constant time, <code>size()</code> will be implemented in linear
	 * time.
	 *
	 * @param inNewA Beginning of sequence of <code>Positions</code> to be
	 * removed from.
	 * @param inNewB End of sequence of <code>Positions</code> to be removed from
	 * this <code>CircularSequence</code>.
	 * @return A new <code>CircularSequence</code> holding <code>inNewA,</code>
	 * <code>inNewB,</code> and all the <code>Positions</code> between them
	 * @exception EmptyContainerException if this container is empty.
	 * @exception InvalidPositionException if <code>inNewA</code> or
	 * <code>inNewB</code> is <code>null</code>, from a different container, or
	 * of an incompatible type.
		* @exception InvalidContainerException if this container has been
		* invalidated.
	 * @see splice
	 * @see jdsl.simple.api.SimpleContainer#size()
	 */
	public CircularSequence split (Position inNewA, Position inNewB)
		throws EmptyContainerException, InvalidPositionException, InvalidContainerException;


	/** 
	 * Merges in another <code>CircularSequence</code> after a given position.
	 * That is, this <code>CircularSequence</code> is snipped after a given
	 * <code>Position,</code> and the merged-in <code>CircularSequence</code> is
	 * snipped before a given Position.  <code>toMerge</code> will be invalidated
	 * after this method returns.  In structures that can implement this method
	 * in constant time, <code>size()</code> will be implemented in linear
	 * time.
	 *
	 * @param afterWhich <code>Position</code> after which to break this
	 * <code>CircularSequence</code>
	 * @param toMerge <code>CircularSequence</code> to be spliced into this one
	 * @param newSuccessor <code>Position</code> from toMerge that will follow
	 * <code>Position</code> afterWhich from this <code>CircularSequence</code>.
	 * @exception InvalidContainerException If this container is invalid or if
	 * <code>toMerge is invalid or of an incompatible type.
	 * @exception InvalidArgumentException if <code>toMerge</code> is the same
	 * container as <code>splice</code> is called on.
	 * @exception InvalidPositionException if <code>afterWhich</code> or
	 * <code>newSuccessor</code> is either null, or of the wrong type.
	 * @see jdsl.simple.api.SimpleContainer#size()
	 */
	public void splice (Position afterWhich,
			CircularSequence toMerge, Position newSuccessor)throws InvalidContainerException, InvalidPositionException, InvalidArgumentException;


} // end of definition of CircularSequence interface
