/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

import java.util.Enumeration;
import jdsl.core.api.*;

/**
 * An interface describing a graph as a combinatorial object.  Holds
 * both directed and undirected edges.
 * 
 * Note that the Vertex and Edge interfaces are subinterfaces of
 * jdsl.core.api.Position -- empty subinterfaces, for type safety only.
 *
 * @author Mike Boilen
 * @version JSDL-Teach release 1.0
 */
public interface Graph extends InspectableGraph {

  /* primitive update methods */

  /**
   * Inserts a new isolated vertex containing an object.
   * @param info the object to be stored in the new vertex
   * @return the new vertex
   */
  public Vertex insertVertex (Object info) throws InvalidPositionException;

  /**
   * Inserts a new undirected edge between two existing vertices.
   * @param u the first endvertex
   * @param v the second endvertex
   * @return the new edge
   */
  public Edge insertEdge (Vertex u, Vertex v, Object info)
     throws InvalidPositionException;

  /**
   * Inserts a new directed edge from an existing vertex to another.
   * @param u the origin vertex
   * @param v the destination vertex
   * @return the new edge
   */
  public Edge insertDirectedEdge (Vertex u, Vertex v, Object info) throws
  InvalidPositionException;

  /**
   * Deletes a vertex and all its incident edges.  If you need the locators
   * stored at the removed edges, get the locators beforehand.<p>
   * @param v the vertex to be deleted
   * @return Locator formerly stored at v
   */
  public Object removeVertex (Vertex v) throws InvalidPositionException;
  
  /**
   * Removes an edge.
   * @param e the edge to be removed
   * @return The removed edge's element
   */
  public Object removeEdge (Edge e) throws InvalidPositionException;
  
  /* other update methods */

  /**
   * Sets the direction of an edge away from a vertex.  Makes an
   * undirected edge directed.
   * @param e an edge
   * @param v an endvertex of e
   * @throws InvalidEdgeException When v is not an endpoint of e
   */
  public void setDirectionFrom (Edge e, Vertex newOrigin) throws
  InvalidEdgeException, InvalidPositionException;

  /**
   * Sets the direction of an edge towards a vertex.  Makes an
   * undirected edge directed.
   * @param e an edge
   * @param v an endvertex of e
   * @throws InvalidEdgeException When v is not an endpoint of e
   */
  public void setDirectionTo (Edge e, Vertex newDestination) throws
  InvalidEdgeException, InvalidPositionException;

  /**
   * Makes a directed edge undirected. Does nothing if the edge is
   * undirected.
   * @param e an edge
   */
  public void makeUndirected (Edge e) throws InvalidEdgeException;

  /**
   * Reverse the direction of an edge.
   * @param e an edge
   * @throws InvalidEdgeException If the edge is undirected
   */
  public void reverseDirection (Edge e) throws InvalidEdgeException,
    InvalidPositionException;

}
   
