/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

import java.util.Enumeration;
import jdsl.core.api.*;



/**
 * An interface describing a graph as a combinatorial object.  Holds
 * both directed and undirected edges.
 * 
 * @author Mike Boilen (mgb)
 * @version JSDL-Teach release 1.0
 */
public interface InspectableGraph extends PositionalContainer {

  

  /* graph methods */

  /**
   * @return the number of vertices
   */
  public int numVertices();

  /**
   * @return the number of edges
   */
  public int numEdges();

  /**
   * @return an enumeration over the vertices
   */
  public Enumeration vertices();

  /**
   * @return an enumeration over the edges
   */
  public Enumeration edges();

  /**
   * @return an enumeration over the directed edges
   */
  public Enumeration directedEdges();

  /**
   * @return an enumeration over the undirected edges
   */
  public Enumeration undirectedEdges();


  
  /* vertex methods */

  /**
   * @param v a vertex
   * @return the number of edges (directed and undirected) incident with v
   */
  public int degree (Vertex v) throws InvalidPositionException;

  /**
   * @param v a vertex
   * @return the number of incoming edges of v
   */
  public int inDegree (Vertex v) throws InvalidPositionException;

  /**
   * @param v a vertex
   * @return the number of outgoing edges of v
   */
  public int outDegree (Vertex v) throws InvalidPositionException;

  /**
   * "Adjacent" here includes directed incoming edges, in addition to
   * the more expected directed outgoing and undirected edges.<p>
   * @param v a vertex
   * @return an enumeration over all vertices adjacent to v
   */
  public Enumeration adjacentVertices (Vertex v) throws
  InvalidPositionException;

  /**
   * @param v a vertex
   * @return an enumeration over the vertices adjacent to v by incoming edges
   */
  public Enumeration inAdjacentVertices (Vertex v) throws
  InvalidPositionException;

  /**
   * @param v a vertex
   * @return an enumeration over the vertices adjacent to v by outgoing edges
   */
  public Enumeration outAdjacentVertices (Vertex v) throws
  InvalidPositionException;

  /**
   * @param v a vertex
   * @return an enumeration over all edges incident with v
   */
  public Enumeration incidentEdges (Vertex v) throws InvalidPositionException;

  /**
   * @param v a vertex
   * @return an enumeration over the incoming edges of v
   */
  public Enumeration inIncidentEdges (Vertex v) throws
  InvalidPositionException;

  /**
   * @param v a vertex
   * @return an enumeration over the outgoing edges of v
   */
  public Enumeration outIncidentEdges (Vertex v) throws
  InvalidPositionException;



  /* edge methods */

  /**
   * @param e an edge
   * @return an enumeration over the two endvertices of e
   */
  public Vertex[] endVertices (Edge e) throws InvalidPositionException;

  /**
   * @param e an edge
   * @param v one endvertex of e
   * @return the endvertex of e different from v
   * @throws InvalidEdgeException When v is not an endpoint of e
   */
  public Vertex opposite (Vertex v, Edge e) throws InvalidEdgeException,
    InvalidPositionException;

  /**
   * @param e an edge
   * @return the origin vertex of e, if e is directed
   * @throws InvalidEdgeException When e is undirected
   */
  public Vertex origin (Edge e) throws InvalidEdgeException,
    InvalidPositionException;

  /**
   * @param e an edge
   * @return the destination vertex of e, if e is directed
   * @throws InvalidEdgeException When e is undirected
   */
  public Vertex destination (Edge e) throws InvalidEdgeException,
    InvalidPositionException;



  /* direction methods */

  /**
   * @param e an edge
   * @return true if e is directed, false otherwise
   */
  public boolean isDirected (Edge e) throws InvalidEdgeException;



}
   

