/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

import java.util.Enumeration;

/**
  * A Container in which each element in its structure is associated with a
  * key.  This association is known as a <I>mapping</I>. A KeyBasedContainer 
  * does not explicitly use an element to determine its internal position. 
  * Instead the key to which the element is mapped determines this position 
  * (this key may be the element itself, however). 
  * <P>
  * KeyBasedContainers may be viewed abstractly as storing pairs of the 
  * type, <I>p(k,e)</I> where
  * <UL>
  *     <LI> <I>k</I> is the key 
  *     <LI> <I>e</I> is the element mapped to <I>k</I>
  * </UL>
  * In the above example, <I>k</I> is known as an <I>explicit</I> key. That is
  * <I>k</I> is some object other than <I>e</I>. In this case <I>k</I> is used
  * to determine <I>e</I>'s position in the container. KeyBasedContainers also 
  * allow the use of an <I>implicit</I> key &shy; where the element itself is 
  * used as a key. The difference between the two forms is seen in the methods 
  * which use them. For example
  * <BLOCKQUOTE><PRE>
  *     Locator insert(Object key, Object element);
  * </PRE></BLOCKQUOTE>
  * is an <I>explicit</I> key method call, whereas
  * <BLOCKQUOTE><PRE>
  *     Locator insert(Object element);
  * </PRE></BLOCKQUOTE>
  * is an <I>implicit</I> key call.
  *
  * @version JSDL-Teach release 1.0
 * @author John Kloss
  */

public interface KeyBasedContainer extends Container {
  
  /**
    * Inserts a Locator into this Container.  
    *
    * @param              locator  The Locator whose key and element are inserted into
    *                     this Container.
    *
    * @exception          InvalidKeyException      If the key referenced
    *                     by <code>locator</code> is not a type accepted by
    *                     this Container.  (For instance, if the container's Comparator
    *                     can't compare the element, or if the container should have
    *                     a comparator but doesn't.)
    * @exception          InvalidLocatorException  If <code>locator</code> 
    *                     cannot be inserted into this container.
    */
  void insert(Locator locator)
    throws InvalidKeyException,
    InvalidLocatorException,ContainedLocatorException;
  
  /**
    * Inserts a &lt;key, element&gt; pair into this Container. Here, <code>
    * key</code> is an <I>explicit</I> key. That is, it is mapped to 
    * <code>element</code> and used to position <code>element</code> within 
    * this Container.
    *
    * @param              key      The key used to position the element
    *                     within this Container.
    * @param              element  The element to be inserted into this
    *                     Container.
    *
    * @return             A Locator which points to <code>element</code> 
    *                     within this Container.
    *
    * @exception          InvalidKeyException  If <code>key</code>
    *                     is not a type accepted by this Container.  (For instance, if
    *                     the container's Comparator
    *                     can't compare the element, or if the container should have
    *                     a comparator but doesn't.)
    */
  Locator insert(Object key, Object element)
    throws InvalidKeyException;
  
  /**
    * Removes an element from this Container. This method provides the only 
    * guaranteed way of removing a particular element from a KeyBasedContainer.
    * The Container may make no guarantee about the uniqueness of its keys or 
    * stored elements. Many elements may be mapped to the same key and vice
    * versa depending upon the particular implementaion. 
    *
    * @param              locator  The Locator which points to a particular
    *                     element within this Container.
    *
    * @exception          InvalidLocatorException  If <code>locator</code> is
    *                     invalid.
		*
		* @exception					UncontainedLocatorException  If <code>locator</code>
		* 										is not in a container.
    */
  void remove(Locator locator)
    throws InvalidLocatorException, UncontainedLocatorException;
  
  /**
    * Changes the mapping of a Locator's element to a new key. That is, within
    * this Container <code>locator</code>'s element will now be mapped to 
    * <code>key</code>. The original key this element was mapped to is returned
    * <B>Note:</B> this method does not necessarily remove the old key (other
    * elements within this Container may be mapped to it).
    *
    * @param              locator  The Locator which points to a particular
    *                     element within this Container.
    * @param              key      The new key to which <code>locator</code>'s
    *                     element should be mapped.
    *
    * @return             The old key to which <code>locator</code>'s element
    *                     was mapped.
    *
    * @exception          InvalidLocatorException  If <code>locator</code> is
    *                     invalid (For example: It does not actually reference
    *                     an element in this Container).
    * @exception          InvalidKeyException      If <code>key</code>
    *                     is not a type accepted by this Container (e.g. If
    *                     this Container is unable to use <code>key</code>
    *                     as a key).
    */
  Object replaceKey(Locator locator, Object key)
    throws InvalidLocatorException,
    InvalidKeyException;
  
  /** 
    * Takes constant time -- even in key-based containers, since the
    * element can be changed independently of the key.
    * 
    * @param loc Locator at which replacement should occur
    * @param newElement Element now to be stored at Locator loc
    * @return Old element, previously stored at Locator loc
    */
  public Object replaceElement (Locator loc, Object newElement) throws
    InvalidLocatorException;
  
  /**
    * Returns an Enumeration of all the Locators within this Container. If 
    * this Container is empty then an empty Enumeration is returned.
    *
    * @return             An Enumeration of all the Locators stored in the
    *                     container. <B>Note:</B> this Enumeration may be
    *                     empty.
    */
  Enumeration locators();

  /**
    * Returns an enumeration of all of the keys of all the locators in the
    * Container.
    *
    * @return An Enumeration of all the keys in this container. <B>Note:</B>
    * this Enumeration may be empty.
    */
  public Enumeration keys();
  
  /**
    * For when you need a locator that can be inserted into this
    * KeyBasedContainer but don't want to insert it quite yet.
    */
  Locator makeLocator (Object key, Object element)throws InvalidKeyException;
}

