/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;

/* A Locator is a coat-check, of sorts, for a client's (key,element)
 * pair.  If you give a Container an element to hold, the Container
 * can give you back a Locator, and you can later ask the Container to
 * do useful things with the Locator.  One thing you can do is have
 * the Container stop storing the element, by passing the element's
 * Locator to the Container's remove(.)  method.  When this happens,
 * the locator becomes free-floating -- not held by any container, but
 * able to be inserted into any container of the same class as the one
 * from which it was removed.  <p>
 * 
 * The implementation of Locators is not specified, beyond this
 * interface.  However, the point of Locators is to allow faster
 * access to arbitrary elements stored in the Container than would be
 * possible by simply searching the Container for them, so they should
 * not be implemented with a simple search.  They might be references
 * to node objects or indices into arrays.<p>
 *
 * A different Locator is associated with each reference to an element
 * that you ask the Container to store.  That is, if you put identical
 * elements into a Container, their Locators will not be the same.
 * However, if the Container moves the element around, the Locator
 * follows the element.  That is, a Locator is associated with one
 * stored reference to an element, no matter where that reference is
 * stored in the Container.<p>
 * 
 * @author Mark Handy (mdh)
 * @version JSDL-Teach release 1.0
 */

public interface Locator {

/* Gets the element associated with the Locator.
 * @return User's element indexed by the Locator
 * @exception InvalidLocatorException if the locator is internally invalid
 */
  public Object element() throws InvalidLocatorException;

  
/* Gets the key associated with the Locator.
 * @return User's key indexed by the Locator
 * @exception InvalidLocatorException if the locator is internally invalid
 */
  public Object key() throws InvalidLocatorException;
    
/* Tests whether the locator refers to an item in a container 
 * @return boolean.
 * @exception InvalidLocatorException if the locator is internally invalid
 */
  public boolean isContained() throws InvalidLocatorException;

/* Gets the Container that 
 * is storing the element associated with the Locator.
 * @return Container holding element indexed by the Locator.
 * @exception InvalidLocatorException if the locator is internally invalid
 * @exception UncontainedLocatorException if the locator is not in a
 * container
 */
  public Container container() throws InvalidLocatorException,
    UncontainedLocatorException;
 
}
