/*
 * Copyright (C) 1998, Center for Geometric Computing, Brown University
 *                 and The Johns Hopkins University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose other than its incorporation into a  
 * commercial product is hereby granted without fee, provided that the
 * above copyright notice appear in all copies and that both that   
 * copyright notice and this permission notice appear in supporting 
 * documentation, and that the names of Brown University and of The Johns
 * Hopkins University not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 * 
 * BROWN UNIVERSITY AND THE JOHNS HOPKINS UNIVERSITY DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR  
 * PURPOSE. IN NO EVENT SHALL BROWN UNIVERSITY OR THE JOHNS HOPKINS
 * UNIVERSITY BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL 
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

package jdsl.core.api;
import jdsl.core.ref.AlwaysInvalidLocator;

/**
 * A Dictionary in which the keys are totally ordered. That is, given a set of
 * keys, <I>S</I>, where <I>a</I> and <I>b</I> are elements of <I>S</I>,
 * exactly one of the following properties holds:
 * <UL>
 *     <LI> <I>a</I> &lt; <I>b</I>
 *     <LI> <I>a</I> = <I>b</I>
 *     <LI> <I>a</I> &gt; <I>b</I>
 * </UL>
 * This expansion of the Dictionary interface allows OrderedDictionaries to be
 * searched through sequentially.
 *
 * @version JSDL-Teach release 1.0
 * @author John Kloss
 */

public interface OrderedDictionary extends Dictionary {

/** 
 * Returned from all four methods of OrderedDictionary to
 * indicate that the user tried to go before the first element
 * of the dictionary or after the last.  Note that dictionaries have
 * special values returned from lookup methods, rather than exceptions,
 * when the lookup fails.
 */
  public static Locator BOUNDARY_VIOLATION = new
    AlwaysInvalidLocator("OrderedDictionary.BOUNDARY_VIOLATION");

    /**
     * Returns the Locator that is sequentially before another Locator in this
     * Container. If there is no such element then the returned Locator will
     * be invalid.
     *
     * @param              locator An abstract position in this Container.
     *
     * @return             A Locator which is sequentially before <code>
     *                     locator</code>. <B>Note:</B> this Locator will be 
     *                     the invalid BOUNDARY_VIOLATION if no such locator exists.
     *
     * @exception          InvalidLocatorException  If <code>locator</code> is
     *                     invalid (For example: It does not actually reference
     *                     an element within this Container).
     */
    Locator before(Locator locator)
	throws InvalidLocatorException, UncontainedLocatorException;

    /**
     * Returns the Locator that is sequentially after another Locator in this
     * Container. If there is no such element then the returned Locator will
     * be invalid.
     *
     * @param              locator An abstract position in this Container.
     *
     * @return             A Locator which is sequentially after <code>
     *                     locator</code>. <B>Note:</B> this Locator will be 
     *                     the invalid BOUNDARY_VIOLATION if no such locator exists.
     *
     * @exception          InvalidLocatorException  If <code>locator</code> is
     *                     invalid (For example: It does not actually reference
     *                     an element within this Container).
     */
    Locator after(Locator locator)
	throws InvalidLocatorException, UncontainedLocatorException;

    /**
     * Returns a Locator whose key is sequentially less than or equal to
     * the search key.  
     * The key which is ordered before <code>key</code> may have many Locators
     * mapped to it. Which one is closest to <code>key</code> is implementation
     * dependent. It is also possible that there are <I>no</I> such Locators
     * ordered before <code>key</code>, in which case, the Locator returned 
     * will be invalid.  Finally, it is possible that <code>key</code>
     * is actually present in the container, in which case a locator whose
     * key is equal to <code>key</code> will be returned.
     *
     * @param              key An abstract position in this Container.
     *
     * @return             The first Locator whose key is less than or
     *                     equal to
     *                     <code>key</code> in this Container. <B>Note:</B>
     *                     If no such Locator exists, the returned Locator will
     *                     be the invalid BOUNDARY_VIOLATION.  
     *
     * @exception          InvalidKeyException  If <code>key</code> is not of
     *                     a type accepted by this Container (For example: The
     *                     key is not comparable).
     */
    Locator closestBefore(Object key)
	throws InvalidKeyException, UncontainedLocatorException;

    /**
     * Returns a Locator whose key is sequentially greater than or
     * equal to the search key. 
     * The key which is ordered after <code>key</code> may have many Locators
     * mapped to it. Which one is closest to <code>key</code> is implementation
     * dependent. It is also possible that there are <I>no</I> such Locators
     * ordered after <code>key</code>, in which case, the Locator returned 
     * will be invalid.  Finally, it is possible that <code>key</code>
     * is actually present in the container, in which case a locator whose
     * key is equal to <code>key</code> will be returned.
     *
     * @param              key An abstract position in this Container.
     *
     * @return             The first Locator whose key is greater than
     *                     or equal to
     *                     <code>key</code> in this Container. <B>Note:</B>
     *                     If no such Locator exists, the returned Locator will
     *                     be the invalid BOUNDARY_VIOLATION.
     *
     * @exception          InvalidKeyException  If <code>key</code> is not of
     *                     a type accepted by this Container (For example: The
     *                     key is not comparable).
     */
    Locator closestAfter(Object key)
	throws InvalidKeyException, UncontainedLocatorException;
}
